﻿
using UnityEngine;
using System.Collections;


public class PlayerControl : MonoBehaviour
{
    private enum Keys : int { Left = 0, Right, Up, Down, Jump, Dash, Shoot };

    // movement config
    public float pushForce = 20f;
    public float gravity = -15f;
    public float runSpeed = 8f;
    public float groundDamping = 20f; // how fast do we change direction? higher means faster
    public float inAirDamping = 5f;
    public float targetJumpHeight = 2f;
    public float lastDashStart = float.NegativeInfinity;
    [HideInInspector]
    public Vector2 dashDirection = Vector2.right;
    [HideInInspector]
    public Vector2 currentDashDirection = Vector2.right;
    public float dashCompletionTime = 0.25f;
    public float dashStartSpeed = 20.0f;
    public float dashEndSpeed = 8.0f;
    public float dashCoolDown = 0.5f;
    public float dashCollisionCoolDown = 1.5f;
    [HideInInspector]
    public float currentDashCoolDown = 0.0f;
    // public float jumpWaitTime = 2.0;

    [HideInInspector]
    public float rawMovementDirection = 1;
    //[HideInInspector]
    public float normalizedHorizontalSpeed = 0;

    CharacterController2D _controller;
    public RaycastHit2D lastControllerColliderHit;

    [HideInInspector]
    public Vector3 velocity;

    public int playerNumber = 1; //gibt an, ob es sich um player one, player two, etc. handelt. sollte nicht 0 sein;
    KeyCode[] keyCodes = new KeyCode[Keys.GetNames(typeof(Keys)).Length];

    void Start()
    {
        _controller = GetComponent<CharacterController2D>();
        _controller.onControllerCollidedEvent += onControllerCollider;

        switch (playerNumber)
        {
            case 1:
                keyCodes[(int)Keys.Right] = KeyCode.D;
                keyCodes[(int)Keys.Left] = KeyCode.A;
                keyCodes[(int)Keys.Up] = KeyCode.W;
                keyCodes[(int)Keys.Jump] = keyCodes[(int)Keys.Up];
                keyCodes[(int)Keys.Down] = KeyCode.S;
                keyCodes[(int)Keys.Dash] = KeyCode.Space;
                keyCodes[(int)Keys.Shoot] = KeyCode.LeftShift;
                break;

            case 2:
                keyCodes[(int)Keys.Right] = KeyCode.RightArrow;
                keyCodes[(int)Keys.Left] = KeyCode.LeftArrow;
                keyCodes[(int)Keys.Up] = KeyCode.UpArrow;
                keyCodes[(int)Keys.Jump] = keyCodes[(int)Keys.Up];
                keyCodes[(int)Keys.Down] = KeyCode.DownArrow;
                keyCodes[(int)Keys.Dash] = KeyCode.RightControl;
                keyCodes[(int)Keys.Shoot] = KeyCode.RightShift;
                break;

            default:
                Debug.LogError ("No keys defined for player number " + playerNumber);
                break;
        }
    }

    void onControllerCollider(RaycastHit2D hit)
    {
        Rigidbody2D body = hit.collider.gameObject.GetComponent<Rigidbody2D>();
        if (body)
        {
            //TODO abfrage " is dashing"
            body.AddForce(_controller.getDirectionOfMovement() * pushForce);
            lastDashStart = Time.time - dashCompletionTime;
            currentDashCoolDown = dashCollisionCoolDown;
        }
    }

    void Update()
    {
        // grab our current velocity to use as a base for all calculations
        velocity = _controller.velocity;

        if (_controller.isGrounded)
            velocity.y = 0;

        updateDashDirection();

        if (canDash () && Input.GetKey (keyCodes[(int)Keys.Dash]))
        {
          lastDashStart = Time.time;
          currentDashDirection = dashDirection;
          currentDashCoolDown = dashCoolDown;
        }

        if (isDashing ())
        {
          velocity = currentDashDirection * Mathf.Lerp(dashStartSpeed, dashEndSpeed, getDashTime() / dashCompletionTime);
        }
        else
        {
          if (Input.GetKey(keyCodes[(int)Keys.Right]))
          {
              normalizedHorizontalSpeed = 1;
              if (transform.localScale.x < 0f)
                  transform.localScale = new Vector3(-transform.localScale.x, transform.localScale.y, transform.localScale.z);

              //if (_controller.isGrounded)
          }
          else if (Input.GetKey(keyCodes[(int)Keys.Left]))
          {
              normalizedHorizontalSpeed = -1;
              if (transform.localScale.x > 0f)
                  transform.localScale = new Vector3(-transform.localScale.x, transform.localScale.y, transform.localScale.z);

              //if (_controller.isGrounded)
          }
          else
          {
              normalizedHorizontalSpeed = 0;

              //if (_controller.isGrounded)
          }

          if (Input.GetKey (keyCodes[(int)Keys.Jump]))
          {
              //to avoid DOUBLE JUMP
              if (_controller.isGrounded)
              {
                  velocity.y = Mathf.Sqrt(2f * targetJumpHeight * -gravity);
              }
          }


          // apply horizontal speed smoothing it
          var smoothedMovementFactor = _controller.isGrounded ? groundDamping : inAirDamping; // how fast do we change direction?
          velocity.x = Mathf.Lerp(velocity.x, normalizedHorizontalSpeed * rawMovementDirection * runSpeed, Time.deltaTime * smoothedMovementFactor);

          // apply gravity before moving
          velocity.y += gravity * Time.deltaTime;

          /*==========Power Ups // Bullet management ===*/
          if (Input.GetKeyDown(keyCodes[(int)Keys.Shoot]))
          {
            Debug.Log("Shoot pressed. Can shoot: " + canShoot());
            if (canShoot())
            {
              shoot();
              coolDown();
            }
          }
        }

        _controller.move(velocity * Time.deltaTime);
    }

    private void updateDashDirection()
    {
      float horizontalDashDirection = 0.0f;
      if (Input.GetKey (keyCodes[(int)Keys.Right]))
      {
        horizontalDashDirection = 1.0f;
      }
      else if (Input.GetKey (keyCodes[(int)Keys.Left]))
      {
        horizontalDashDirection = -1.0f;
      }

      float verticalDashDirection = 0.0f;
      if (Input.GetKey (keyCodes[(int)Keys.Up]))
      {
        verticalDashDirection = 1.0f;
      }
      else if (Input.GetKey (keyCodes[(int)Keys.Down]))
      {
        verticalDashDirection = -1.0f;
      }

      if (horizontalDashDirection != 0.0f || verticalDashDirection != 0.0f)
        dashDirection = new Vector2(horizontalDashDirection, verticalDashDirection);
    }

    private float getDashTime ()
    {
        return Time.time - lastDashStart;
    }

    private bool isDashing ()
    {
        float dashTime = getDashTime();
        return 0 <= dashTime && dashTime < dashCompletionTime;
    }

    private bool canDash ()
    {
        return getDashTime() >= dashCompletionTime + currentDashCoolDown;
    }

    /*==========Power Ups // Bullet management ===*/

    public const int POWERUP_NONE = 0;
    public const int POWERUP_HEALINGBULLET = 1;
    public const int POWERUP_TRAPDESTROYER = 2;

    private int powerUpType = POWERUP_NONE;

    private bool coolingDown = false;

    public float coolDownTime = 2.0f;

    public GameObject healingBulletPrefab;
    public GameObject trapDestroyingBulletPrefab;

    public float spawnDistance = 1.0f;
    public void setPowerUpType(int type)
    {
        powerUpType = type;

    }

    private void shoot()
    {
        float dir = Mathf.Sign(transform.localScale.x);
        Vector2 pos = new Vector2(transform.position.x + dir * spawnDistance, transform.position.y);

        switch (powerUpType)
        {
            case POWERUP_HEALINGBULLET:
                GameObject bullet = Instantiate(healingBulletPrefab, pos, transform.rotation) as GameObject;
                bullet.GetComponent<Bullet>().shoot(new Vector2(dir, 0));
                break;
            case POWERUP_TRAPDESTROYER:
                GameObject trapbullet = Instantiate(trapDestroyingBulletPrefab, pos, transform.rotation) as GameObject;
                trapbullet.GetComponent<Bullet>().shoot(new Vector2(dir, 0));
                break;
            default:
                break;
        }
    }

    private void coolDown()
    {
        coolingDown = true;
        Invoke("coolDownOver", coolDownTime);
    }

    private void coolDownOver()
    {
        coolingDown = false;
    }

    private bool canShoot()
    {
        return (!coolingDown) && powerUpType != POWERUP_NONE;
    }

}
